#!/usr/bin/python
#
# Copyright (c) 2017 Sertac Ozercan <seozerca@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_virtualmachineextension

version_added: "0.1.2"

short_description: Managed Azure Virtual Machine extension

description:
    - Create, update and delete Azure Virtual Machine Extension.
    - Note that this module was called M(azure.azcollection.azure_rm_virtualmachine_extension) before Ansible 2.8. The usage did not change.

options:
    resource_group:
        description:
            - Name of a resource group where the vm extension exists or will be created.
        required: true
        type: str
    name:
        description:
            - Name of the vm extension.
        required: true
        type: str
    state:
        description:
            - State of the vm extension. Use C(present) to create or update a vm extension and C(absent) to delete a vm extension.
        default: present
        type: str
        choices:
            - absent
            - present
    location:
        description:
            - Valid Azure location. Defaults to location of the resource group.
        type: str
    virtual_machine_name:
        description:
            - The name of the virtual machine where the extension should be create or updated.
        required: true
        type: str
    publisher:
        description:
            - The name of the extension handler publisher.
        type: str
    virtual_machine_extension_type:
        description:
            - The type of the extension handler.
        type: str
    type_handler_version:
        description:
            - The type version of the extension handler.
        type: str
    settings:
        description:
            - JSON formatted public settings for the extension.
        type: dict
    protected_settings:
        description:
            - JSON formatted protected settings for the extension.
            - >-
                Previously configured settings are not available, so the parameter is not used for idempotency checks.
                If changes to this parameter need to be applied, use in conjunction with I(force_update_tag).
        type: dict
    auto_upgrade_minor_version:
        description:
            - Whether the extension handler should be automatically upgraded across minor versions during deployment.
        type: bool
    enable_automatic_upgrade:
        description:
            - Whether the extension handler should be automatically upgraded.
        type: bool
    force_update_tag:
        description:
            - Whether the extension should be updated or re-run even if no changes can be detected from what is currently configured.
            - Helpful when applying changes to I(protected_settings).
        type: bool
        default: false
        version_added: '1.10.0'

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Sertac Ozercan (@sozercan)
    - Julien Stroheker (@julienstroheker)
'''

EXAMPLES = '''
- name: Create VM Extension
  azure_rm_virtualmachineextension:
    name: myvmextension
    location: eastus
    resource_group: myResourceGroup
    virtual_machine_name: myvm
    publisher: Microsoft.Azure.Extensions
    virtual_machine_extension_type: CustomScript
    type_handler_version: 2.0
    settings: '{"commandToExecute": "hostname"}'
    auto_upgrade_minor_version: true

- name: Delete VM Extension
  azure_rm_virtualmachineextension:
    name: myvmextension
    resource_group: myResourceGroup
    virtual_machine_name: myvm
    state: absent
'''

RETURN = '''
state:
    description:
        - Current state of the vm extension.
    returned: always
    type: dict
    sample: { "state":"Deleted" }

changed:
    description:
        - Whether or not the resource has changed.
    returned: always
    type: bool
    sample: true
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


def vmextension_to_dict(extension):
    '''
    Serializing the VM Extension from the API to Dict
    :return: dict
    '''
    return dict(
        id=extension.id,
        name=extension.name,
        location=extension.location,
        publisher=extension.publisher,
        virtual_machine_extension_type=extension.type_properties_type,
        type_handler_version=extension.type_handler_version,
        auto_upgrade_minor_version=extension.auto_upgrade_minor_version,
        enable_automatic_upgrade=extension.enable_automatic_upgrade,
        settings=extension.settings,
        protected_settings=extension.protected_settings,
    )


class AzureRMVMExtension(AzureRMModuleBase):
    """Configuration class for an Azure RM VM Extension resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
            location=dict(
                type='str'
            ),
            virtual_machine_name=dict(
                type='str',
                required=True
            ),
            publisher=dict(
                type='str'
            ),
            virtual_machine_extension_type=dict(
                type='str'
            ),
            type_handler_version=dict(
                type='str'
            ),
            auto_upgrade_minor_version=dict(
                type='bool'
            ),
            enable_automatic_upgrade=dict(
                type='bool'
            ),
            settings=dict(
                type='dict'
            ),
            protected_settings=dict(
                type='dict', no_log=True
            ),
            force_update_tag=dict(
                type='bool',
                default=False
            ),
        )

        self.resource_group = None
        self.name = None
        self.virtual_machine_name = None
        self.location = None
        self.publisher = None
        self.virtual_machine_extension_type = None
        self.type_handler_version = None
        self.auto_upgrade_minor_version = None
        self.enable_automatic_upgrade = None
        self.settings = None
        self.protected_settings = None
        self.state = None
        self.force_update_tag = False

        required_if = [
            ('state', 'present', ['publisher', 'virtual_machine_extension_type', 'type_handler_version']),
        ]

        self.results = dict(changed=False, state=dict())

        super(AzureRMVMExtension, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=False,
                                                 supports_tags=False,
                                                 required_if=required_if)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        resource_group = None
        to_be_updated = False

        resource_group = self.get_resource_group(self.resource_group)
        if not self.location:
            self.location = resource_group.location

        response = self.get_vmextension()

        if self.state == 'present':
            if not response:
                to_be_updated = True
            else:
                if self.force_update_tag:
                    to_be_updated = True

                if self.settings is not None:
                    if response['settings'] != self.settings:
                        response['settings'] = self.settings
                        to_be_updated = True
                else:
                    self.settings = response['settings']

                if response['location'] != self.location:
                    self.location = response['location']
                    self.module.warn("Property 'location' cannot be changed")

                if response['publisher'] != self.publisher:
                    self.publisher = response['publisher']
                    self.module.warn("Property 'publisher' cannot be changed")

                if response['virtual_machine_extension_type'] != self.virtual_machine_extension_type:
                    self.virtual_machine_extension_type = response['virtual_machine_extension_type']
                    self.module.warn("Property 'virtual_machine_extension_type' cannot be changed")

                if response['type_handler_version'] != self.type_handler_version:
                    response['type_handler_version'] = self.type_handler_version
                    to_be_updated = True

                if self.auto_upgrade_minor_version is not None:
                    if response['auto_upgrade_minor_version'] != self.auto_upgrade_minor_version:
                        response['auto_upgrade_minor_version'] = self.auto_upgrade_minor_version
                        to_be_updated = True
                else:
                    self.auto_upgrade_minor_version = response['auto_upgrade_minor_version']

                if self.auto_upgrade_minor_version is not None:
                    if response['enable_automatic_upgrade'] != self.enable_automatic_upgrade:
                        response['enable_automatic_upgrade'] = self.enable_automatic_upgrade
                        to_be_updated = True
                else:
                    self.enable_automatic_upgrade = response['enable_automatic_upgrade']

            if to_be_updated:
                self.results['changed'] = True
                self.results['state'] = self.create_or_update_vmextension()
        elif self.state == 'absent':
            if response:
                self.delete_vmextension()
                self.results['changed'] = True

        return self.results

    def create_or_update_vmextension(self):
        '''
        Method calling the Azure SDK to create or update the VM extension.
        :return: void
        '''
        self.log("Creating VM extension {0}".format(self.name))
        try:
            params = self.compute_models.VirtualMachineExtension(
                location=self.location,
                publisher=self.publisher,
                type_properties_type=self.virtual_machine_extension_type,
                type_handler_version=self.type_handler_version,
                auto_upgrade_minor_version=self.auto_upgrade_minor_version,
                enable_automatic_upgrade=self.enable_automatic_upgrade,
                settings=self.settings,
                protected_settings=self.protected_settings,
                force_update_tag=self.force_update_tag,
            )
            poller = self.compute_client.virtual_machine_extensions.begin_create_or_update(self.resource_group, self.virtual_machine_name, self.name, params)
            response = self.get_poller_result(poller)
            return vmextension_to_dict(response)

        except Exception as e:
            self.log('Error attempting to create the VM extension.')
            self.fail("Error creating the VM extension: {0}".format(str(e)))

    def delete_vmextension(self):
        '''
        Method calling the Azure SDK to delete the VM Extension.
        :return: void
        '''
        self.log("Deleting vmextension {0}".format(self.name))
        try:
            poller = self.compute_client.virtual_machine_extensions.begin_delete(self.resource_group, self.virtual_machine_name, self.name)
            self.get_poller_result(poller)
        except Exception as e:
            self.log('Error attempting to delete the vmextension.')
            self.fail("Error deleting the vmextension: {0}".format(str(e)))

    def get_vmextension(self):
        '''
        Method calling the Azure SDK to get a VM Extension.
        :return: void
        '''
        self.log("Checking if the vm extension {0} is present".format(self.name))
        found = False
        try:
            response = self.compute_client.virtual_machine_extensions.get(self.resource_group, self.virtual_machine_name, self.name)
            found = True
        except ResourceNotFoundError as e:
            self.log('Did not find vm extension')
        if found:
            return vmextension_to_dict(response)
        else:
            return False


def main():
    """Main execution"""
    AzureRMVMExtension()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_domain
short_description: Manage UCS Domain Profiles in Cisco Intersight
description:
  - Create, update, and delete UCS Domain Profiles (SwitchClusterProfiles) on Cisco Intersight.
  - Manages the associated SwitchProfiles (A and B), Fabric Interconnect assignments, and policy buckets.
  - Policies are attached to SwitchProfiles via the Intersight bulk API.
  - This module does not manage deployment of the domain profile.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/fabric/SwitchClusterProfiles/post/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the UCS Domain Profile.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
      - SwitchProfiles are automatically named C({name}-A) and C({name}-B).
    type: str
    required: true
  description:
    description:
      - The user-defined description of the UCS Domain Profile.
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  assigned_switch_a_serial:
    description:
      - The serial number of the Fabric Interconnect to assign to SwitchProfile A.
      - Resolved via the C(/network/ElementSummaries) API endpoint using the C(Serial) field.
    type: str
  assigned_switch_b_serial:
    description:
      - The serial number of the Fabric Interconnect to assign to SwitchProfile B.
      - Resolved via the C(/network/ElementSummaries) API endpoint using the C(Serial) field.
    type: str
  vlan_policy_fi_a:
    description:
      - Name of the VLAN Policy to associate with Fabric Interconnect A.
    type: str
  vlan_policy_fi_b:
    description:
      - Name of the VLAN Policy to associate with Fabric Interconnect B.
    type: str
  vsan_policy_fi_a:
    description:
      - Name of the VSAN Policy to associate with Fabric Interconnect A.
    type: str
  vsan_policy_fi_b:
    description:
      - Name of the VSAN Policy to associate with Fabric Interconnect B.
    type: str
  port_policy_fi_a:
    description:
      - Name of the Port Policy to associate with Fabric Interconnect A.
    type: str
  port_policy_fi_b:
    description:
      - Name of the Port Policy to associate with Fabric Interconnect B.
    type: str
  ntp_policy:
    description:
      - Name of the NTP Policy to associate with both Fabric Interconnects.
    type: str
  syslog_policy:
    description:
      - Name of the Syslog Policy to associate with both Fabric Interconnects.
    type: str
  network_connectivity_policy:
    description:
      - Name of the Network Connectivity (DNS) Policy to associate with both Fabric Interconnects.
    type: str
  snmp_policy:
    description:
      - Name of the SNMP Policy to associate with both Fabric Interconnects.
    type: str
  ldap_policy:
    description:
      - Name of the LDAP Policy to associate with both Fabric Interconnects.
    type: str
  certificate_management_policy:
    description:
      - Name of the Certificate Management Policy to associate with both Fabric Interconnects.
    type: str
  system_qos_policy:
    description:
      - Name of the System QoS Policy to associate with both Fabric Interconnects.
      - This policy is mandatory for UCS Domain Profiles.
    type: str
  auditd_policy:
    description:
      - Name of the Audit Log Policy to associate with both Fabric Interconnects.
    type: str
  switch_control_policy:
    description:
      - Name of the Switch Control Policy to associate with both Fabric Interconnects.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a basic UCS Domain Profile
  cisco.intersight.intersight_domain:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "Domain-01"
    description: "Basic domain profile"
    system_qos_policy: "Default-QoS"
    state: present

- name: Create a UCS Domain Profile with switch assignments and policies
  cisco.intersight.intersight_domain:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "Domain-01"
    description: "Full domain profile"
    assigned_switch_a_serial: "FDO23456ABC"
    assigned_switch_b_serial: "FDO23456DEF"
    vlan_policy_fi_a: "VLAN-Policy-A"
    vlan_policy_fi_b: "VLAN-Policy-B"
    vsan_policy_fi_a: "VSAN-Policy-A"
    vsan_policy_fi_b: "VSAN-Policy-B"
    port_policy_fi_a: "Port-Policy-A"
    port_policy_fi_b: "Port-Policy-B"
    ntp_policy: "NTP-Corp"
    syslog_policy: "Syslog-Corp"
    snmp_policy: "SNMP-Monitor"
    system_qos_policy: "QoS-Default"
    switch_control_policy: "Switch-Control"
    state: present

- name: Delete a UCS Domain Profile
  cisco.intersight.intersight_domain:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Domain-01"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the SwitchClusterProfile resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "Domain-01",
        "ObjectType": "fabric.SwitchClusterProfile",
        "Organization": {
            "Moid": "675450ee69726530014753e2",
            "ObjectType": "organization.Organization"
        },
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import (
    IntersightModule,
    intersight_argument_spec,
    resolve_policy_bucket,
    sync_policy_bucket,
)


PER_FI_A_POLICY_MAPPING = {
    'vlan_policy_fi_a': {'resource_path': '/fabric/EthNetworkPolicies', 'object_type': 'fabric.EthNetworkPolicy'},
    'vsan_policy_fi_a': {'resource_path': '/fabric/FcNetworkPolicies', 'object_type': 'fabric.FcNetworkPolicy'},
    'port_policy_fi_a': {'resource_path': '/fabric/PortPolicies', 'object_type': 'fabric.PortPolicy'},
}

PER_FI_B_POLICY_MAPPING = {
    'vlan_policy_fi_b': {'resource_path': '/fabric/EthNetworkPolicies', 'object_type': 'fabric.EthNetworkPolicy'},
    'vsan_policy_fi_b': {'resource_path': '/fabric/FcNetworkPolicies', 'object_type': 'fabric.FcNetworkPolicy'},
    'port_policy_fi_b': {'resource_path': '/fabric/PortPolicies', 'object_type': 'fabric.PortPolicy'},
}

SHARED_POLICY_MAPPING = {
    'system_qos_policy': {'resource_path': '/fabric/SystemQosPolicies', 'object_type': 'fabric.SystemQosPolicy'},
    'ntp_policy': {'resource_path': '/ntp/Policies', 'object_type': 'ntp.Policy'},
    'syslog_policy': {'resource_path': '/syslog/Policies', 'object_type': 'syslog.Policy'},
    'network_connectivity_policy': {'resource_path': '/networkconfig/Policies', 'object_type': 'networkconfig.Policy'},
    'snmp_policy': {'resource_path': '/snmp/Policies', 'object_type': 'snmp.Policy'},
    'ldap_policy': {'resource_path': '/iam/LdapPolicies', 'object_type': 'iam.LdapPolicy'},
    'certificate_management_policy': {'resource_path': '/certificatemanagement/Policies', 'object_type': 'certificatemanagement.Policy'},
    'auditd_policy': {'resource_path': '/auditd/Policies', 'object_type': 'auditd.Policy'},
    'switch_control_policy': {'resource_path': '/fabric/SwitchControlPolicies', 'object_type': 'fabric.SwitchControlPolicy'},
}


def resolve_switch_moid_by_serial(intersight, serial):
    """Resolve a Fabric Interconnect serial number to its MOID via /network/ElementSummaries."""
    intersight.get_resource(
        resource_path='/network/ElementSummaries',
        query_params={
            '$filter': f"Serial eq '{serial}'",
            '$select': 'Moid',
        },
    )
    moid = intersight.result['api_response'].get('Moid')
    if not moid:
        intersight.module.fail_json(
            msg=f"Fabric Interconnect with serial '{serial}' not found."
        )
    return moid


def ensure_switch_profile(intersight, domain_name, switch_id, cluster_moid):
    """Ensure a SwitchProfile exists for the given switch ID, return its MOID."""
    profile_name = f"{domain_name}-{switch_id}"
    filter_str = (
        f"Name eq '{profile_name}'"
        f" and SwitchClusterProfile.Moid eq '{cluster_moid}'"
    )

    intersight.get_resource(
        resource_path='/fabric/SwitchProfiles',
        query_params={'$filter': filter_str},
    )

    if intersight.result['api_response'].get('Moid'):
        return intersight.result['api_response']['Moid']

    body = {
        'Name': profile_name,
        'SwitchId': switch_id,
        'SwitchClusterProfile': {'Moid': cluster_moid},
    }
    intersight.configure_resource(
        moid=None,
        resource_path='/fabric/SwitchProfiles',
        body=body,
        query_params={'$filter': filter_str},
    )
    return intersight.result['api_response'].get('Moid')


def configure_switch_profiles(intersight, domain_name, cluster_moid):
    """Create SwitchProfiles A/B and assign Fabric Interconnects if provided.

    Returns a list of (profile_moid, profile_state) tuples for each FI.
    """
    fi_configs = [
        {'switch_id': 'A', 'switch_param': 'assigned_switch_a_serial'},
        {'switch_id': 'B', 'switch_param': 'assigned_switch_b_serial'},
    ]

    profile_results = []
    for fi in fi_configs:
        profile_moid = ensure_switch_profile(intersight, domain_name, fi['switch_id'], cluster_moid)
        profile_state = dict(intersight.result['api_response'])

        switch_serial = intersight.module.params.get(fi['switch_param'])
        if switch_serial and profile_moid:
            switch_moid = resolve_switch_moid_by_serial(intersight, switch_serial)
            current_switch = (profile_state.get('AssignedSwitch') or {}).get('Moid')
            if current_switch != switch_moid:
                intersight.configure_resource(
                    moid=profile_moid,
                    resource_path='/fabric/SwitchProfiles',
                    body={'AssignedSwitch': {'Moid': switch_moid}},
                    query_params={},
                )

        profile_results.append((profile_moid, profile_state))

    return profile_results


def build_policy_sync_data(intersight, organization, profile_results):
    """Resolve desired vs current PolicyBuckets for each SwitchProfile.

    Returns a list of (profile_moid, desired_bucket, current_bucket) tuples
    ready for sync_domain_policy_buckets.
    """
    per_fi_mappings = [PER_FI_A_POLICY_MAPPING, PER_FI_B_POLICY_MAPPING]
    shared_bucket = resolve_policy_bucket(intersight, organization, SHARED_POLICY_MAPPING)

    profiles_to_sync = []
    for (profile_moid, profile_state), per_fi_mapping in zip(profile_results, per_fi_mappings):
        if not profile_moid:
            continue
        desired_bucket = resolve_policy_bucket(intersight, organization, per_fi_mapping) + shared_bucket
        current_bucket = profile_state.get('PolicyBucket') or []
        profiles_to_sync.append((profile_moid, desired_bucket, current_bucket))

    return profiles_to_sync


def unassign_switches_before_delete(intersight, cluster_moid):
    """Unassign switches from SwitchProfiles before deleting the domain.

    The Intersight API may prevent deletion of a domain profile while
    switches are assigned to its SwitchProfiles.
    """
    intersight.get_resource(
        resource_path='/fabric/SwitchProfiles',
        query_params={
            '$filter': f"SwitchClusterProfile.Moid eq '{cluster_moid}'"
        },
        return_list=True,
    )

    profiles = intersight.result['api_response']
    if not isinstance(profiles, list):
        profiles = [profiles] if profiles else []

    for profile in profiles:
        if not profile.get('Moid'):
            continue
        assigned = profile.get('AssignedSwitch')
        if assigned and assigned.get('Moid'):
            intersight.configure_resource(
                moid=profile['Moid'],
                resource_path='/fabric/SwitchProfiles',
                body={'AssignedSwitch': None},
                query_params={},
            )


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        assigned_switch_a_serial=dict(type='str'),
        assigned_switch_b_serial=dict(type='str'),
        vlan_policy_fi_a=dict(type='str'),
        vlan_policy_fi_b=dict(type='str'),
        vsan_policy_fi_a=dict(type='str'),
        vsan_policy_fi_b=dict(type='str'),
        port_policy_fi_a=dict(type='str'),
        port_policy_fi_b=dict(type='str'),
        ntp_policy=dict(type='str'),
        syslog_policy=dict(type='str'),
        network_connectivity_policy=dict(type='str'),
        snmp_policy=dict(type='str'),
        ldap_policy=dict(type='str'),
        certificate_management_policy=dict(type='str'),
        system_qos_policy=dict(type='str'),
        auditd_policy=dict(type='str'),
        switch_control_policy=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'present', ['system_qos_policy']],
        ],
        required_together=[
            ['assigned_switch_a_serial', 'assigned_switch_b_serial'],
        ],
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    cluster_path = '/fabric/SwitchClusterProfiles'
    name = intersight.module.params['name']
    organization = intersight.module.params['organization']
    state = intersight.module.params['state']

    intersight.api_body = {
        'Organization': {'Name': organization},
        'Name': name,
        'TargetPlatform': 'UCS Domain',
    }

    if state == 'present':
        intersight.set_tags_and_description()

    if state == 'absent':
        organization_moid = intersight.get_moid_by_name(
            resource_path='/organization/Organizations',
            resource_name=organization,
        )
        if organization_moid:
            filter_str = (
                f"Name eq '{name}'"
                f" and Organization.Moid eq '{organization_moid}'"
            )
            intersight.get_resource(
                resource_path=cluster_path,
                query_params={'$filter': filter_str},
            )
            cluster_moid = intersight.result['api_response'].get('Moid')
            if cluster_moid:
                unassign_switches_before_delete(intersight, cluster_moid)

    cluster_moid = intersight.configure_policy_or_profile(resource_path=cluster_path)
    cluster_response = dict(intersight.result['api_response']) if intersight.result['api_response'] else {}

    if cluster_moid and state == 'present':
        profile_results = configure_switch_profiles(intersight, name, cluster_moid)
        profiles_to_sync = build_policy_sync_data(intersight, organization, profile_results)
        for profile_moid, desired_bucket, current_bucket in profiles_to_sync:
            bucket_path = f'/fabric/SwitchProfiles/{profile_moid}/PolicyBucket'
            sync_policy_bucket(intersight, bucket_path, desired_bucket, current_bucket)

    if cluster_response:
        intersight.result['api_response'] = cluster_response

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2020, FELDSAM s.r.o. - FeldHost™ <support@feldhost.cz>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: opennebula
author:
  - Kristian Feldsam (@feldsam)
short_description: OpenNebula inventory source
version_added: "3.8.0"
extends_documentation_fragment:
  - ansible.builtin.constructed
description:
  - Get inventory hosts from OpenNebula cloud.
  - Uses an YAML configuration file ending with either C(opennebula.yml) or C(opennebula.yaml) to set parameter values.
  - Uses O(api_authfile), C(~/.one/one_auth), or E(ONE_AUTH) pointing to a OpenNebula credentials file.
options:
  plugin:
    description: Token that ensures this is a source file for the 'opennebula' plugin.
    type: string
    required: true
    choices: [community.general.opennebula]
  api_url:
    description:
      - URL of the OpenNebula RPC server.
      - It is recommended to use HTTPS so that the username/password are not transferred over the network unencrypted.
      - If not set then the value of the E(ONE_URL) environment variable is used.
    env:
      - name: ONE_URL
    required: true
    type: string
  api_username:
    description:
      - Name of the user to login into the OpenNebula RPC server. If not set then the value of the E(ONE_USERNAME) environment
        variable is used.
    env:
      - name: ONE_USERNAME
    type: string
  api_password:
    description:
      - Password or a token of the user to login into OpenNebula RPC server.
      - If not set, the value of the E(ONE_PASSWORD) environment variable is used.
    env:
      - name: ONE_PASSWORD
    required: false
    type: string
  api_authfile:
    description:
      - If both O(api_username) or O(api_password) are not set, then it tries to authenticate with ONE auth file. Default
        path is C(~/.one/one_auth).
      - Set environment variable E(ONE_AUTH) to override this path.
    env:
      - name: ONE_AUTH
    required: false
    type: string
  hostname:
    description: Field to match the hostname. Note V(v4_first_ip) corresponds to the first IPv4 found on VM.
    type: string
    default: v4_first_ip
    choices:
      - v4_first_ip
      - v6_first_ip
      - name
  filter_by_label:
    description: Only return servers filtered by this label.
    type: string
  group_by_labels:
    description: Create host groups by VM labels.
    type: bool
    default: true
"""

EXAMPLES = r"""
# inventory_opennebula.yml file in YAML format
# Example command line: ansible-inventory --list -i inventory_opennebula.yml

# Pass a label filter to the API
plugin: community.general.opennebula
api_url: https://opennebula:2633/RPC2
filter_by_label: Cache
"""

try:
    import pyone

    HAS_PYONE = True
except ImportError:
    HAS_PYONE = False

import os
from dataclasses import dataclass

from ansible.errors import AnsibleError
from ansible.plugins.inventory import BaseInventoryPlugin, Constructable

from ansible_collections.community.general.plugins.plugin_utils._unsafe import make_unsafe


@dataclass
class AuthParams:
    url: str
    username: str
    password: str


class InventoryModule(BaseInventoryPlugin, Constructable):
    NAME = "community.general.opennebula"

    def verify_file(self, path):
        valid = False
        if super().verify_file(path):
            if path.endswith(("opennebula.yaml", "opennebula.yml")):
                valid = True
        return valid

    def _get_connection_info(self):
        url = self.get_option("api_url")
        username = self.get_option("api_username")
        password = self.get_option("api_password")
        authfile = self.get_option("api_authfile")

        if not username and not password:
            if authfile is None:
                authfile = os.path.join(os.environ.get("HOME"), ".one", "one_auth")
            try:
                with open(authfile) as fp:
                    authstring = fp.read().rstrip()
                username, password = authstring.split(":")
            except OSError as e:
                raise AnsibleError(f"Could not find or read ONE_AUTH file at '{authfile}'") from e
            except Exception as e:
                raise AnsibleError(f"Error occurs when reading ONE_AUTH file at '{authfile}'") from e

        return AuthParams(url=url, username=username, password=password)

    def _get_vm_ipv4(self, vm):
        nic = vm.TEMPLATE.get("NIC")

        if isinstance(nic, dict):
            nic = [nic]

        for net in nic:
            if net.get("IP"):
                return net["IP"]

        return False

    def _get_vm_ipv6(self, vm):
        nic = vm.TEMPLATE.get("NIC")

        if isinstance(nic, dict):
            nic = [nic]

        for net in nic:
            if net.get("IP6_GLOBAL"):
                return net["IP6_GLOBAL"]

        return False

    def _get_vm_pool(self):
        auth = self._get_connection_info()

        if not (auth.username and auth.password):
            raise AnsibleError("API Credentials missing. Check OpenNebula inventory file.")
        else:
            one_client = pyone.OneServer(auth.url, session=f"{auth.username}:{auth.password}")

        # get hosts (VMs)
        try:
            vm_pool = one_client.vmpool.infoextended(-2, -1, -1, 3)
        except Exception as e:
            raise AnsibleError(f"Something happened during XML-RPC call: {e}") from e

        return vm_pool

    def _retrieve_servers(self, label_filter=None):
        vm_pool = self._get_vm_pool()

        result = []

        # iterate over hosts
        for vm in vm_pool.VM:
            server = vm.USER_TEMPLATE

            labels = []
            if vm.USER_TEMPLATE.get("LABELS"):
                labels = [
                    s for s in vm.USER_TEMPLATE.get("LABELS") if s == "," or s == "-" or s.isalnum() or s.isspace()
                ]
                labels = "".join(labels)
                labels = labels.replace(" ", "_")
                labels = labels.replace("-", "_")
                labels = labels.split(",")

            # filter by label
            if label_filter is not None:
                if label_filter not in labels:
                    continue

            server["name"] = vm.NAME
            server["id"] = vm.ID
            if hasattr(vm.HISTORY_RECORDS, "HISTORY") and vm.HISTORY_RECORDS.HISTORY:
                server["host"] = vm.HISTORY_RECORDS.HISTORY[-1].HOSTNAME
            server["LABELS"] = labels
            server["v4_first_ip"] = self._get_vm_ipv4(vm)
            server["v6_first_ip"] = self._get_vm_ipv6(vm)

            result.append(server)

        return result

    def _populate(self):
        hostname_preference = self.get_option("hostname")
        group_by_labels = self.get_option("group_by_labels")
        strict = self.get_option("strict")

        # Add a top group 'one'
        self.inventory.add_group(group="all")

        filter_by_label = self.get_option("filter_by_label")
        servers = self._retrieve_servers(filter_by_label)
        for server in servers:
            server = make_unsafe(server)
            hostname = server["name"]
            # check for labels
            if group_by_labels and server["LABELS"]:
                for label in server["LABELS"]:
                    self.inventory.add_group(group=label)
                    self.inventory.add_host(host=hostname, group=label)

            self.inventory.add_host(host=hostname, group="all")

            for attribute, value in server.items():
                self.inventory.set_variable(hostname, attribute, value)

            if hostname_preference != "name":
                self.inventory.set_variable(hostname, "ansible_host", server[hostname_preference])

            if server.get("SSH_PORT"):
                self.inventory.set_variable(hostname, "ansible_port", server["SSH_PORT"])

            # handle construcable implementation: get composed variables if any
            self._set_composite_vars(self.get_option("compose"), server, hostname, strict=strict)

            # groups based on jinja conditionals get added to specific groups
            self._add_host_to_composed_groups(self.get_option("groups"), server, hostname, strict=strict)

            # groups based on variables associated with them in the inventory
            self._add_host_to_keyed_groups(self.get_option("keyed_groups"), server, hostname, strict=strict)

    def parse(self, inventory, loader, path, cache=True):
        if not HAS_PYONE:
            raise AnsibleError("OpenNebula Inventory plugin requires pyone to work!")

        super().parse(inventory, loader, path)
        self._read_config_data(path=path)

        self._populate()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from .._import_helper import HTTPAdapter, urllib3
from .basehttpadapter import BaseHTTPAdapter

# Resolves OpenSSL issues in some servers:
#   https://lukasa.co.uk/2013/01/Choosing_SSL_Version_In_Requests/
#   https://github.com/kennethreitz/requests/pull/799


PoolManager = urllib3.poolmanager.PoolManager


class SSLHTTPAdapter(BaseHTTPAdapter):
    """An HTTPS Transport Adapter that uses an arbitrary SSL version."""

    __attrs__ = HTTPAdapter.__attrs__ + ["assert_hostname"]

    def __init__(
        self,
        assert_hostname: bool | None = None,
        **kwargs: t.Any,
    ) -> None:
        self.assert_hostname = assert_hostname
        super().__init__(**kwargs)

    def init_poolmanager(
        self, connections: int, maxsize: int, block: bool = False, **kwargs: t.Any
    ) -> None:
        kwargs = {
            "num_pools": connections,
            "maxsize": maxsize,
            "block": block,
        }
        if self.assert_hostname is not None:
            kwargs["assert_hostname"] = self.assert_hostname

        self.poolmanager = PoolManager(**kwargs)

    def get_connection(self, *args: t.Any, **kwargs: t.Any) -> urllib3.ConnectionPool:
        """
        Ensure assert_hostname is set correctly on our pool

        We already take care of a normal poolmanager via init_poolmanager

        But we still need to take care of when there is a proxy poolmanager

        Note that this method is no longer called for newer requests versions.
        """
        # pylint finds our HTTPAdapter stub instead of requests.adapters.HTTPAdapter:
        # pylint: disable-next=no-member
        conn = super().get_connection(*args, **kwargs)
        if (
            self.assert_hostname is not None
            and conn.assert_hostname != self.assert_hostname  # type: ignore
        ):
            conn.assert_hostname = self.assert_hostname  # type: ignore
        return conn



# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Kevin Breit (@kbreit) <kevin.breit@kevinbreit.net>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


class ModuleDocFragment(object):
    # Standard files for documentation fragment
    DOCUMENTATION = r'''
notes:
- More information about the Meraki API can be found at U(https://dashboard.meraki.com/api_docs).
- Some of the options are likely only used for developers within Meraki.
- As of Ansible 2.9, Meraki modules output keys as snake case. To use camel case, set the C(ANSIBLE_MERAKI_FORMAT) environment variable to C(camelcase).
- Ansible's Meraki modules will stop supporting camel case output in Ansible 2.13. Please update your playbooks.
- Check Mode downloads the current configuration from the dashboard, then compares changes against this download. Check Mode will report changed if
    there are differences in the configurations, but does not submit changes to the API for validation of change.
options:
    auth_key:
        description:
        - Authentication key provided by the dashboard. Required if environmental variable C(MERAKI_KEY) is not set.
        type: str
        required: true
    host:
        description:
        - Hostname for Meraki dashboard.
        - Can be used to access regional Meraki environments, such as China.
        type: str
        default: api.meraki.com
    use_proxy:
        description:
        - If C(no), it will not use a proxy, even if one is defined in an environment variable on the target hosts.
        type: bool
        default: false
    use_https:
        description:
        - If C(no), it will use HTTP. Otherwise it will use HTTPS.
        - Only useful for internal Meraki developers.
        type: bool
        default: true
    output_format:
        description:
        - Instructs module whether response keys should be snake case (ex. C(net_id)) or camel case (ex. C(netId)).
        type: str
        choices: [snakecase, camelcase]
        default: snakecase
    output_level:
        description:
        - Set amount of debug output during module execution.
        type: str
        choices: [ debug, normal ]
        default: normal
    timeout:
        description:
        - Time to timeout for HTTP requests.
        type: int
        default: 30
    validate_certs:
        description:
        - Whether to validate HTTP certificates.
        type: bool
        default: true
    org_name:
        description:
        - Name of organization.
        type: str
        aliases: [ organization ]
    org_id:
        description:
        - ID of organization.
        type: str
    rate_limit_retry_time:
        description:
        - Number of seconds to retry if rate limiter is triggered.
        type: int
        default: 165
    internal_error_retry_time:
        description:
        - Number of seconds to retry if server returns an internal server error.
        type: int
        default: 60
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_camera_detections_history_by_boundary_by_interval_info
short_description: Information module for organizations _camera _detections _history
  _by _boundary _by _interval
description:
  - Get all organizations _camera _detections _history _by _boundary _by _interval.
  - Returns analytics data for timespans.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  boundaryIds:
    description:
      - >
        BoundaryIds query parameter. A list of boundary ids. The returned cameras
        will be filtered to only include these ids.
    elements: str
    type: list
  ranges:
    description:
      - Ranges query parameter. A list of time ranges with intervals.
    elements: dict
    type: list
  duration:
    description:
      - >
        Duration query parameter. The minimum time, in seconds, that the person or
        car remains in the area to be counted. Defaults to boundary configuration
        or 60.
    type: int
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 1 - 1000. Defaults to 1000.
    type: int
  boundaryTypes:
    description:
      - BoundaryTypes query parameter. The detection types. Defaults to 'person'.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for camera getOrganizationCameraDetectionsHistoryByBoundaryByInterval
    description: Complete reference of the getOrganizationCameraDetectionsHistoryByBoundaryByInterval
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-camera-detections-history-by-boundary-by-interval
notes:
  - SDK Method used are
    camera.Camera.get_organization_camera_detections_history_by_boundary_by_interval,
  - Paths used are
    get /organizations/{organizationId}/camera/detections/history/byBoundary/byInterval,
"""

EXAMPLES = r"""
- name: Get all organizations _camera _detections _history _by _boundary _by _interval
  cisco.meraki.organizations_camera_detections_history_by_boundary_by_interval_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    boundaryIds: []
    ranges: []
    duration: 0
    perPage: 0
    boundaryTypes: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "boundaryId": "string",
        "type": "string",
        "results": {
          "startTime": "string",
          "endTime": "string",
          "objectType": "string",
          "in": 0,
          "out": 0
        }
      }
    ]
"""



#!/usr/bin/python
#
# Copyright (c) 2020 Nikhil Patne (@nikhilpatne) and Sakar Mehra (@sakar97)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_apimanagementservice_info
version_added: '1.5.0'
short_description: Get ApiManagementService info
description:
    - Get info of ApiManagementService.
options:
    resource_group:
        description:
            - The name of the resource group.
        type: str
    name:
        description:
            - Resource name.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure
author:
    - Nikhil Patne (@nikhilpatne)
    - Sakar Mehra (@Sakar97)

'''

EXAMPLES = '''
- name: Get Api Management Service By Name and Resource Group
  azure_rm_apimanagementservice_info:
    resource_group: myResourceGroup
    name: myApiName

- name: Get Api Management Service By Resource Group
  azure_rm_apimanagementservice_info:
    resource_group: myResourceGroup

- name: Get Api Management Service By Subscription
  azure_rm_apimanagementservice_info:
'''

RETURN = '''
api_management_service:
    description:
        - A list of dict results where the key is the name of the ApiManagementService.
        - The values are the facts for that ApiManagementService.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxx-xxx/resourceGroups/myResourceGroup/providers/Microsoft.ApiManagement/service/myPolicy
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: myPolicy
        type:
            description:
                - Resource type for API Management resource is set to Microsoft.ApiManagement.
            returned: always
            type: str
            sample: Microsoft.ApiManagement/service
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: {'key1':'value1'}
        properties:
            description:
                - Properties of the API Management service.
            returned: always
            type: dict
            sample: null
        location:
            description:
                - Resource location.
            type: str
            returned: always
            sample: 'East US'
        sku:
            description:
                - SKU properties of the API Management service.
            returned: always
            type: str
            sample: Developer
        etag:
            description:
                - ETag of the resource.
            returned: always
            type: str
            sample: AAAAAAAsQK8=
        zones:
            description:
                - Zone of the resource.
            type: str
            returned: always
            sample: null
        identity:
            description:
                - Identity of the resource.
            type: str
            returned: always
            sample: null
'''

import json
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt


class AzureRMApiManagementServiceInfo(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str'
            ),
            name=dict(
                type='str'
            )
        )

        self.resource_group = None
        self.name = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200]

        self.query_parameters = {}
        self.query_parameters['api-version'] = '2022-08-01'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        self.mgmt_client = None
        super(AzureRMApiManagementServiceInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        if (self.resource_group is not None and self.name is not None):
            self.results['api_management_service'] = self.get()
        elif (self.resource_group is not None):
            self.results['api_management_service'] = self.listbyresourcegroup()
        else:
            self.results['api_management_service'] = self.list()
        return self.results

    def get(self):
        response = None
        results = {}
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.ApiManagement' +
                    '/service' +
                    '/{{ service_name }}')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)
        self.url = self.url.replace('{{ service_name }}', self.name)

        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return self.format_item(results)

    def listbyresourcegroup(self):
        response = None
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.ApiManagement' +
                    '/service')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)

        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return [self.format_item(x) for x in results['value']] if results['value'] else []

    def list(self):
        response = None
        results = {}
        # prepare url
        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/providers' +
                    '/Microsoft.ApiManagement' +
                    '/service')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)

        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            results = json.loads(response.body())
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return [self.format_item(x) for x in results['value']] if results['value'] else []

    def format_item(self, item):
        if item:
            d = {
                'id': item['id'],
                'name': item['name'],
                'type': item['type'],
                'sku': item['sku']['name'],
                'identity': item['identity'],
                'zones': item['zones'],
                'location': item['location'],
                'etag': item['etag'],
                'properties': item['properties']
            }
        else:
            return dict()
        return d


def main():
    AzureRMApiManagementServiceInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2025, Klention Mali <klention@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: lvm_pv_move_data
short_description: Move data between LVM Physical Volumes (PVs)
version_added: "11.2.0"
description:
  - Moves data from one LVM Physical Volume (PV) to another.
author:
  - Klention Mali (@klention)
options:
  source:
    description:
      - Path to the source block device to move data from.
      - Must be an existing PV.
    type: path
    required: true
  destination:
    description:
      - Path to the destination block device to move data to.
      - Must be an existing PV with enough free space.
    type: path
    required: true
  auto_answer:
    description:
      - Answer yes to all prompts automatically.
    type: bool
    default: false
  atomic:
    description:
      - Makes the C(pvmove) operation atomic, ensuring that all affected LVs are moved to the destination PV,
        or none are if the operation is aborted.
    type: bool
    default: true
  autobackup:
    description:
      - Automatically backup metadata before changes (strongly advised!).
    type: bool
    default: true
requirements:
  - LVM2 utilities
  - Both O(source) and O(destination) devices must exist, and the PVs must be in the same volume group.
  - The O(destination) PV must have enough free space to accommodate the O(source) PV's allocated extents.
  - Verbosity is automatically controlled by Ansible's verbosity level (using multiple C(-v) flags).
"""

EXAMPLES = r"""
- name: Moving data from /dev/sdb to /dev/sdc
  community.general.lvm_pv_move_data:
    source: /dev/sdb
    destination: /dev/sdc
"""

RETURN = r"""
actions:
  description: List of actions performed during module execution.
  returned: success
  type: list
  elements: str
  sample: [
    "moved data from /dev/sdb to /dev/sdc",
    "no allocated extents to move",
    "would move data from /dev/sdb to /dev/sdc"
  ]
"""


import os

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._lvm import pvmove_runner, pvs_runner


def main():
    module = AnsibleModule(
        argument_spec=dict(
            source=dict(type="path", required=True),
            destination=dict(type="path", required=True),
            auto_answer=dict(type="bool", default=False),
            atomic=dict(type="bool", default=True),
            autobackup=dict(type="bool", default=True),
        ),
        supports_check_mode=True,
    )

    pvs = pvs_runner(module)
    pvmove = pvmove_runner(module)

    source = module.params["source"]
    destination = module.params["destination"]
    changed = False
    actions = []
    result = {"changed": False}

    # Validate device existence
    if not os.path.exists(source):
        module.fail_json(msg=f"Source device {source} not found")
    if not os.path.exists(destination):
        module.fail_json(msg=f"Destination device {destination} not found")
    if source == destination:
        module.fail_json(msg="Source and destination devices must be different")

    def run_pvs_field(field, device):
        with pvs("noheadings fields devices") as ctx:
            rc, out, err = ctx.run(fields=field, devices=[device])
            if rc != 0:
                module.fail_json(msg=f"Command failed: {err}", stdout=out, stderr=err, rc=rc, cmd=ctx.cmd)
            return out.strip()

    def is_pv(device):
        with pvs("noheadings readonly devices", check_rc=False) as ctx:
            rc, dummy, dummy = ctx.run(devices=[device])
        return rc == 0

    if not is_pv(source):
        module.fail_json(msg=f"Source device {source} is not a PV")
    if not is_pv(destination):
        module.fail_json(msg=f"Destination device {destination} is not a PV")

    vg_src = run_pvs_field("vg_name", source)
    vg_dest = run_pvs_field("vg_name", destination)
    if vg_src != vg_dest:
        module.fail_json(
            msg=f"Source and destination must be in the same VG. Source VG: '{vg_src}', Destination VG: '{vg_dest}'."
        )

    def get_allocated_pe(device):
        try:
            return int(run_pvs_field("pv_pe_alloc_count", device))
        except ValueError:
            module.fail_json(msg=f"Invalid allocated PE count for device {device}")

    allocated = get_allocated_pe(source)
    if allocated == 0:
        actions.append("no allocated extents to move")
    else:
        # Check destination has enough free space
        def get_total_pe(device):
            try:
                return int(run_pvs_field("pv_pe_count", device))
            except ValueError:
                module.fail_json(msg=f"Invalid total PE count for device {device}")

        def get_free_pe(device):
            return get_total_pe(device) - get_allocated_pe(device)

        free_pe_dest = get_free_pe(destination)
        if free_pe_dest < allocated:
            module.fail_json(
                msg=(
                    f"Destination device {destination} has only {int(free_pe_dest)} free physical extents, but "
                    f"source device {source} has {int(allocated)} allocated extents. Not enough space."
                )
            )

        if module.check_mode:
            changed = True
            actions.append(f"would move data from {source} to {destination}")
        else:
            with pvmove("auto_answer atomic autobackup verbosity source destination") as ctx:
                dummy, out, err = ctx.run(verbosity=module._verbosity)
                result["stdout"] = out
                result["stderr"] = err

            changed = True
            actions.append(f"moved data from {source} to {destination}")

    result["changed"] = changed
    result["actions"] = actions
    if actions:
        result["msg"] = f"PV data move: {', '.join(actions)}"
    else:
        result["msg"] = f"No data to move from {source}"

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2021, quidame <quidame@poivron.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: filesize

short_description: Create a file with a given size, or resize it if it exists

description:
  - This module is a simple wrapper around C(dd) to create, extend or truncate a file, given its size. It can be used to manage
    swap files (that require contiguous blocks) or alternatively, huge sparse files.
author:
  - quidame (@quidame)

version_added: "3.0.0"

attributes:
  check_mode:
    support: full
  diff_mode:
    support: full

options:
  path:
    description:
      - Path of the regular file to create or resize.
    type: path
    required: true
  size:
    description:
      - Requested size of the file.
      - The value is a number (either C(int) or C(float)) optionally followed by a multiplicative suffix, that can be one
        of V(B) (bytes), V(KB) or V(kB) (= 1000B), V(MB) or V(mB) (= 1000kB), V(GB) or V(gB) (= 1000MB), and so on for V(T),
        V(P), V(E), V(Z) and V(Y); or alternatively one of V(K), V(k) or V(KiB) (= 1024B); V(M), V(m) or V(MiB) (= 1024KiB);
        V(G), V(g) or V(GiB) (= 1024MiB); and so on.
      - If the multiplicative suffix is not provided, the value is treated as an integer number of blocks of O(blocksize)
        bytes each (float values are rounded to the closest integer).
      - When the O(size) value is equal to the current file size, does nothing.
      - When the O(size) value is bigger than the current file size, bytes from O(source) (if O(sparse) is not V(false)) are
        appended to the file without truncating it, in other words, without modifying the existing bytes of the file.
      - When the O(size) value is smaller than the current file size, it is truncated to the requested value without modifying
        bytes before this value.
      - That means that a file of any arbitrary size can be grown to any other arbitrary size, and then resized down to its
        initial size without modifying its initial content.
    type: raw
    required: true
  blocksize:
    description:
      - Size of blocks, in bytes if not followed by a multiplicative suffix.
      - The numeric value (before the unit) B(MUST) be an integer (or a C(float) if it equals an integer).
      - If not set, the size of blocks is guessed from the OS and commonly results in V(512) or V(4096) bytes, that is used
        internally by the module or when O(size) has no unit.
    type: raw
  source:
    description:
      - Device or file that provides input data to provision the file.
      - This parameter is ignored when O(sparse=true).
    type: path
    default: /dev/zero
  force:
    description:
      - Whether or not to overwrite the file if it exists, in other words, to truncate it from 0. When V(true), the module
        is not idempotent, that means it always reports RV(ignore:changed=true).
      - O(force=true) and O(sparse=true) are mutually exclusive.
    type: bool
    default: false
  sparse:
    description:
      - Whether or not the file to create should be a sparse file.
      - This option is effective only on newly created files, or when growing a file, only for the bytes to append.
      - This option is not supported on OSes or filesystems not supporting sparse files.
      - O(force=true) and O(sparse=true) are mutually exclusive.
    type: bool
    default: false
  unsafe_writes:
    description:
      - This option is silently ignored. This module always modifies file size in-place.
requirements:
  - dd (Data Duplicator) in PATH

extends_documentation_fragment:
  - ansible.builtin.files
  - community.general._attributes

seealso:
  - name: dd(1) manpage for Linux
    description: Manual page of the GNU/Linux's dd implementation (from GNU coreutils).
    link: https://man7.org/linux/man-pages/man1/dd.1.html

  - name: dd(1) manpage for IBM AIX
    description: Manual page of the IBM AIX's dd implementation.
    link: https://www.ibm.com/support/knowledgecenter/ssw_aix_72/d_commands/dd.html

  - name: dd(1) manpage for Mac OSX
    description: Manual page of the Mac OSX's dd implementation.
    link: https://www.unix.com/man-page/osx/1/dd/

  - name: dd(1M) manpage for Solaris
    description: Manual page of the Oracle Solaris's dd implementation.
    link: https://docs.oracle.com/cd/E36784_01/html/E36871/dd-1m.html

  - name: dd(1) manpage for FreeBSD
    description: Manual page of the FreeBSD's dd implementation.
    link: https://www.freebsd.org/cgi/man.cgi?dd(1)

  - name: dd(1) manpage for OpenBSD
    description: Manual page of the OpenBSD's dd implementation.
    link: https://man.openbsd.org/dd

  - name: dd(1) manpage for NetBSD
    description: Manual page of the NetBSD's dd implementation.
    link: https://man.netbsd.org/dd.1

  - name: busybox(1) manpage for Linux
    description: Manual page of the GNU/Linux's busybox, that provides its own dd implementation.
    link: https://www.unix.com/man-page/linux/1/busybox
"""

EXAMPLES = r"""
- name: Create a file of 1G filled with null bytes
  community.general.filesize:
    path: /var/bigfile
    size: 1G

- name: Extend the file to 2G (2*1024^3)
  community.general.filesize:
    path: /var/bigfile
    size: 2G

- name: Reduce the file to 2GB (2*1000^3)
  community.general.filesize:
    path: /var/bigfile
    size: 2GB

- name: Fill a file with random bytes for backing a LUKS device
  community.general.filesize:
    path: ~/diskimage.luks
    size: 512.0 MiB
    source: /dev/urandom

- name: Take a backup of MBR boot code into a file, overwriting it if it exists
  community.general.filesize:
    path: /media/sdb1/mbr.bin
    size: 440B
    source: /dev/sda
    force: true

- name: Create/resize a sparse file of/to 8TB
  community.general.filesize:
    path: /var/local/sparsefile
    size: 8TB
    sparse: true

- name: Create a file with specific size and attributes, to be used as swap space
  community.general.filesize:
    path: /var/swapfile
    size: 2G
    blocksize: 512B
    mode: u=rw,go=
    owner: root
    group: root
"""

RETURN = r"""
cmd:
  description: Command executed to create or resize the file.
  type: str
  returned: when changed or failed
  sample: /usr/bin/dd if=/dev/zero of=/var/swapfile bs=1048576 seek=3072 count=1024

filesize:
  description: Dictionary of sizes related to the file.
  type: dict
  returned: always
  contains:
    blocks:
      description: Number of blocks in the file.
      type: int
      sample: 500
    blocksize:
      description: Size of the blocks in bytes.
      type: int
      sample: 1024
    bytes:
      description: Size of the file, in bytes, as the product of RV(filesize.blocks) and RV(filesize.blocksize).
      type: int
      sample: 512000
    iec:
      description: Size of the file, in human-readable format, following IEC standard.
      type: str
      sample: 500.0 KiB
    si:
      description: Size of the file, in human-readable format, following SI standard.
      type: str
      sample: 512.0 kB

size_diff:
  description: Difference (positive or negative) between old size and new size, in bytes.
  type: int
  sample: -1234567890
  returned: always

path:
  description: Realpath of the file if it is a symlink, otherwise the same than module's param.
  type: str
  sample: /var/swap0
  returned: always
"""


import math
import os
import re

from ansible.module_utils.basic import AnsibleModule

# These are the multiplicative suffixes understood (or returned) by dd and
# others (ls, df, lvresize, lsblk...).
SIZE_UNITS = dict(
    B=1,
    kB=1000**1,
    KB=1000**1,
    KiB=1024**1,
    K=1024**1,
    k=1024**1,
    MB=1000**2,
    mB=1000**2,
    MiB=1024**2,
    M=1024**2,
    m=1024**2,
    GB=1000**3,
    gB=1000**3,
    GiB=1024**3,
    G=1024**3,
    g=1024**3,
    TB=1000**4,
    tB=1000**4,
    TiB=1024**4,
    T=1024**4,
    t=1024**4,
    PB=1000**5,
    pB=1000**5,
    PiB=1024**5,
    P=1024**5,
    p=1024**5,
    EB=1000**6,
    eB=1000**6,
    EiB=1024**6,
    E=1024**6,
    e=1024**6,
    ZB=1000**7,
    zB=1000**7,
    ZiB=1024**7,
    Z=1024**7,
    z=1024**7,
    YB=1000**8,
    yB=1000**8,
    YiB=1024**8,
    Y=1024**8,
    y=1024**8,
)


def bytes_to_human(size, iec=False):
    """Return human-readable size (with SI or IEC suffix) from bytes. This is
    only to populate the returned result of the module, not to handle the
    file itself (we only rely on bytes for that).
    """
    unit = "B"
    for u, v in SIZE_UNITS.items():
        if size < v:
            continue
        if iec:
            if "i" not in u or size / v >= 1024:
                continue
        else:
            if v % 5 or size / v >= 1000:
                continue
        unit = u

    hsize = round(size / SIZE_UNITS[unit], 2)
    if unit == "B":
        hsize = int(hsize)

    unit = re.sub(r"^(.)", lambda m: m.expand(r"\1").upper(), unit)
    if unit == "KB":
        unit = "kB"

    return f"{hsize} {unit}"


def smart_blocksize(size, unit, product, bsize):
    """Ensure the total size can be written as blocks*blocksize, with blocks
    and blocksize being integers.
    """
    if not product % bsize:
        return bsize

    # Basically, for a file of 8kB (=8000B), system's block size of 4096 bytes
    # is not usable. The smallest integer number of kB to work with 512B blocks
    # is 64, the nexts are 128, 192, 256, and so on.

    unit_size = SIZE_UNITS[unit]

    if size == int(size):
        if unit_size > SIZE_UNITS["MiB"]:
            if unit_size % 5:
                return SIZE_UNITS["MiB"]
            return SIZE_UNITS["MB"]
        return unit_size

    if unit == "B":
        raise AssertionError("byte is the smallest unit and requires an integer value")

    if 0 < product < bsize:
        return product

    for bsz in (1024, 1000, 512, 256, 128, 100, 64, 32, 16, 10, 8, 4, 2):
        if not product % bsz:
            return bsz
    return 1


def split_size_unit(string, isint=False):
    """Split a string between the size value (int or float) and the unit.
    Support optional space(s) between the numeric value and the unit.
    """
    unit = re.sub(r"(\d|\.)", r"", string).strip()
    value = float(re.sub(unit, r"", string).strip())
    if isint and unit in ("B", ""):
        if int(value) != value:
            raise AssertionError("invalid blocksize value: bytes require an integer value")

    if not unit:
        unit = None
        product = int(round(value))
    else:
        if unit not in SIZE_UNITS.keys():
            raise AssertionError(
                f"invalid size unit ({unit}): unit must be one of {', '.join(sorted(SIZE_UNITS, key=SIZE_UNITS.get))}, or none."
            )
        product = int(round(value * SIZE_UNITS[unit]))
    return value, unit, product


def size_string(value):
    """Convert a raw value to a string, but only if it is an integer, a float
    or a string itself.
    """
    if not isinstance(value, (int, float, str)):
        raise AssertionError(f"invalid value type ({type(value)}): size must be integer, float or string")
    return str(value)


def size_spec(args):
    """Return a dictionary with size specifications, especially the size in
    bytes (after rounding it to an integer number of blocks).
    """
    blocksize_in_bytes = split_size_unit(args["blocksize"], True)[2]
    if blocksize_in_bytes == 0:
        raise AssertionError("block size cannot be equal to zero")

    size_value, size_unit, size_result = split_size_unit(args["size"])
    if not size_unit:
        blocks = int(math.ceil(size_value))
    else:
        blocksize_in_bytes = smart_blocksize(size_value, size_unit, size_result, blocksize_in_bytes)
        blocks = int(math.ceil(size_result / blocksize_in_bytes))

    args["size_diff"] = round_bytes = int(blocks * blocksize_in_bytes)
    args["size_spec"] = dict(
        blocks=blocks,
        blocksize=blocksize_in_bytes,
        bytes=round_bytes,
        iec=bytes_to_human(round_bytes, True),
        si=bytes_to_human(round_bytes),
    )
    return args["size_spec"]


def current_size(args):
    """Return the size of the file at the given location if it exists, or None."""
    path = args["path"]
    if os.path.exists(path):
        if not os.path.isfile(path):
            raise AssertionError(f"{path} exists but is not a regular file")
        args["file_size"] = os.stat(path).st_size
    else:
        args["file_size"] = None
    return args["file_size"]


def complete_dd_cmdline(args, dd_cmd):
    """Compute dd options to grow or truncate a file."""
    if args["file_size"] == args["size_spec"]["bytes"] and not args["force"]:
        # Nothing to do.
        return list()

    bs = args["size_spec"]["blocksize"]

    # For sparse files (create, truncate, grow): write count=0 block.
    if args["sparse"]:
        seek = args["size_spec"]["blocks"]
    elif args["force"] or not os.path.exists(args["path"]):  # Create file
        seek = 0
    elif args["size_diff"] < 0:  # Truncate file
        seek = args["size_spec"]["blocks"]
    elif args["size_diff"] % bs:  # Grow file
        seek = int(args["file_size"] / bs) + 1
    else:
        seek = int(args["file_size"] / bs)

    count = args["size_spec"]["blocks"] - seek
    dd_cmd += [f"bs={bs}", f"seek={seek}", f"count={count}"]

    return dd_cmd


def main():
    module = AnsibleModule(
        argument_spec=dict(
            path=dict(type="path", required=True),
            size=dict(type="raw", required=True),
            blocksize=dict(type="raw"),
            source=dict(type="path", default="/dev/zero"),
            sparse=dict(type="bool", default=False),
            force=dict(type="bool", default=False),
        ),
        supports_check_mode=True,
        add_file_common_args=True,
    )
    args = dict(**module.params)
    diff = dict(before=dict(), after=dict())

    if args["sparse"] and args["force"]:
        module.fail_json(msg="parameters values are mutually exclusive: force=true|sparse=true")
    if not os.path.exists(os.path.dirname(args["path"])):
        module.fail_json(msg="parent directory of the file must exist prior to run this module")
    if not args["blocksize"]:
        args["blocksize"] = str(os.statvfs(os.path.dirname(args["path"])).f_frsize)

    try:
        args["size"] = size_string(args["size"])
        args["blocksize"] = size_string(args["blocksize"])
        initial_filesize = current_size(args)
        size_descriptors = size_spec(args)
    except AssertionError as err:
        module.fail_json(msg=f"{err}")

    expected_filesize = size_descriptors["bytes"]
    if initial_filesize:
        args["size_diff"] = expected_filesize - initial_filesize
    diff["after"]["size"] = expected_filesize
    diff["before"]["size"] = initial_filesize

    result = dict(changed=args["force"], size_diff=args["size_diff"], path=args["path"], filesize=size_descriptors)

    dd_bin = module.get_bin_path("dd", True)
    dd_cmd = [dd_bin, f"if={args['source']}", f"of={args['path']}"]

    if expected_filesize != initial_filesize or args["force"]:
        result["cmd"] = " ".join(complete_dd_cmdline(args, dd_cmd))
        if module.check_mode:
            result["changed"] = True
        else:
            result["rc"], dummy, result["stderr"] = module.run_command(dd_cmd)

            diff["after"]["size"] = result_filesize = result["size_diff"] = current_size(args)
            if initial_filesize:
                result["size_diff"] = result_filesize - initial_filesize
            if not args["force"]:
                result["changed"] = result_filesize != initial_filesize

            if result["rc"]:
                msg = f"dd error while creating file {args['path']} with size {args['size']} from source {args['source']}: see stderr for details"
                module.fail_json(msg=msg, **result)
            if result_filesize != expected_filesize:
                msg = (
                    f"module error while creating file {args['path']} with size {args['size']} "
                    f"from source {args['source']}: file is {result_filesize} bytes long"
                )
                module.fail_json(msg=msg, **result)

    # dd follows symlinks, and so does this module, while file module doesn't.
    # If we call it, this is to manage file's mode, owner and so on, not the
    # symlink's ones.
    file_params = dict(**module.params)
    if os.path.islink(args["path"]):
        file_params["path"] = result["path"] = os.path.realpath(args["path"])

    if args["file_size"] is not None:
        file_args = module.load_file_common_arguments(file_params)
        result["changed"] = module.set_fs_attributes_if_different(file_args, result["changed"], diff=diff)
    result["diff"] = diff

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_info
short_description: Information module for organizations _devices
description:
  - Get all organizations _devices.
  - List the devices in an organization that have been assigned to a network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 5000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  configurationUpdatedAfter:
    description:
      - >
        ConfigurationUpdatedAfter query parameter. Filter results by whether or not
        the device's configuration has been updated after the given timestamp.
    type: str
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter devices by network.
    elements: str
    type: list
  productTypes:
    description:
      - >
        ProductTypes query parameter. Optional parameter to filter devices by product
        type. Valid types are wireless, appliance, switch, systemsManager, camera,
        cellularGateway, sensor, wirelessController, campusGateway, and secureConnect.
    elements: str
    type: list
  tags:
    description:
      - Tags query parameter. Optional parameter to filter devices by tags.
    elements: str
    type: list
  tagsFilterType:
    description:
      - >
        TagsFilterType query parameter. Optional parameter of value 'withAnyTags'
        or 'withAllTags' to indicate whether to return networks which contain ANY
        or ALL of the included tags. If no type is included, 'withAnyTags' will be
        selected.
    type: str
  name:
    description:
      - >
        Name query parameter. Optional parameter to filter devices by name. All returned
        devices will have a name that contains the search term or is an exact match.
    type: str
  mac:
    description:
      - >
        Mac query parameter. Optional parameter to filter devices by MAC address.
        All returned devices will have a MAC address that contains the search term
        or is an exact match.
    type: str
  serial:
    description:
      - >
        Serial query parameter. Optional parameter to filter devices by serial number.
        All returned devices will have a serial number that contains the search term
        or is an exact match.
    type: str
  model:
    description:
      - >
        Model query parameter. Optional parameter to filter devices by model. All
        returned devices will have a model that contains the search term or is an
        exact match.
    type: str
  macs:
    description:
      - >
        Macs query parameter. Optional parameter to filter devices by one or more
        MAC addresses. All returned devices will have a MAC address that is an exact
        match.
    elements: str
    type: list
  serials:
    description:
      - >
        Serials query parameter. Optional parameter to filter devices by one or more
        serial numbers. All returned devices will have a serial number that is an
        exact match.
    elements: str
    type: list
  sensorMetrics:
    description:
      - >
        SensorMetrics query parameter. Optional parameter to filter devices by the
        metrics that they provide. Only applies to sensor devices.
    elements: str
    type: list
  sensorAlertProfileIds:
    description:
      - >
        SensorAlertProfileIds query parameter. Optional parameter to filter devices
        by the alert profiles that are bound to them. Only applies to sensor devices.
    elements: str
    type: list
  models:
    description:
      - >
        Models query parameter. Optional parameter to filter devices by one or more
        models. All returned devices will have a model that is an exact match.
    elements: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationDevices
    description: Complete reference of the getOrganizationDevices API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices,
  - Paths used are
    get /organizations/{organizationId}/devices,
"""

EXAMPLES = r"""
- name: Get all organizations _devices
  cisco.meraki.organizations_devices_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    configurationUpdatedAfter: string
    networkIds: []
    productTypes: []
    tags: []
    tagsFilterType: string
    name: string
    mac: string
    serial: string
    model: string
    macs: []
    serials: []
    sensorMetrics: []
    sensorAlertProfileIds: []
    models: []
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "name": "string",
        "lat": 0,
        "lng": 0,
        "address": "string",
        "notes": "string",
        "tags": [
          "string"
        ],
        "networkId": "string",
        "serial": "string",
        "model": "string",
        "imei": "string",
        "mac": "string",
        "lanIp": "string",
        "firmware": "string",
        "productType": "string",
        "details": [
          {
            "name": "string",
            "value": "string"
          }
        ]
      }
    ]
"""
